/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.action;

import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchPart;

import vocleditor.action.request.VOCLExpNegateRequest;
import vocleditor.commands.property.VOCLExpNegateCommand;
import vocleditor.edit.AbstractExpressionEditPart;



/**
 * This action provides the negation of a boolean Expression as well as of an navigation expression.
 */
public class VOCLExpNegateAction extends SelectionAction
{
   

	// The command type as string value
    private final String commandType;

    // The name shown in the Context-Menu
    public static final String NAGATE_PROPERTY = "un/negate";



    /**
     * Creates a new add condition action instance.
     * @param part 
     * @param commandType
     */
    public VOCLExpNegateAction(IWorkbenchPart part, String commandType){
        super(part);
        this.commandType = commandType;
        setId(commandType);
        setText(commandType);
		setImageDescriptor(ImageDescriptor.createFromFile(VOCLExpNegateAction.class,"gif/negate.gif"));
    }

    /**
     * Calculates if this action is enabled, i.e. if the user should
     * be able to execute this action. This is the case, if
     * a command for this action can be instanciated (AddConditiionCommand)
     * and the command can be executed (cmd.canExecute()). 
     */
    protected boolean calculateEnabled(){
        Command cmd = createVOCLExpNegateCommand(getSelectedObjects());
        if (cmd == null)
            return false; 
        return cmd.canExecute();
    }

    /**
     * Creates the coresponding command. This only is done, if a boolean or navigation
     * container is selected in the work-bench.
     * @param list - the list of selected edit-parts
     * @return - an VOCLExpNegateCommand
     */
    protected Command createVOCLExpNegateCommand(List selectedEditParts){
    	// Feature is only available for NavExpContainer and BoolExpression 
        if (selectedEditParts.size() != 1)return null;
        
        // get the EditPart
		AbstractExpressionEditPart ep = null;
        
        // check the EditPart for
        // only allowed for VOCLExpression
        if (!(selectedEditParts.get(0) instanceof AbstractExpressionEditPart)) {
        	return null;
        }else { 
			ep = (AbstractExpressionEditPart)selectedEditParts.get(0);
       
        	// Create the Request	
        	VOCLExpNegateRequest req = new VOCLExpNegateRequest(getCommandType());
			// Create and return the command
			VOCLExpNegateCommand cmd = null;
			if(ep!=null){
				if (ep.understandsRequest(req)) {
					Command c = ep.getCommand(req);
					if (c instanceof VOCLExpNegateCommand) {
						cmd = (VOCLExpNegateCommand)c;
						cmd.setContainer(ep.getExpression());				
					}
				}
			}else 
				System.out.println("Negation of VOCLExpression was not possible, because the Container wasn't there.");
        	
        	return cmd;
        }
    }

    /**
     * Returns the command type.
     * @return the command type
     */
    public String getCommandType(){
        return commandType;
    }

    /**
     * Simply calls this.execute() with the constructor
     */
    public void run(){
        execute(createVOCLExpNegateCommand(getSelectedObjects()));
    }

    /**
     * Initializes this action.
     */
    protected void init(){
        super.init();
        setText(getCommandType());
        setToolTipText(getCommandType());
        setId(getCommandType());
        setEnabled(false);
    }
    
	

}
