/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.connect;

import org.eclipse.gef.commands.Command;

import vocleditor.model.VOCLModelManager;
import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationRole;
import VOCLModel.ClassifierNode;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLModel4Factory;


/**
 * Class       :ConnectCommand<br/>
 * Package     : vocleditor.commands.connect<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Command to connect two ClassifierRoles with AssociationRole
 * <br/>
 * @author vilapower
 */
public abstract class ConnectCommand extends Command {
	
	protected int numberOfSourceAnchor;
	protected int numberOfTargetAnchor;
		

	protected ClassifierNode source;
	protected ClassifierNode target;
	protected AssociationRole association;
	protected NavExpContainer container;
	protected AssociationEndRoleSource assERS ;
	protected AssociationEndRoleTarget assERT ;
	protected VOCLModel4Factory mfc = VOCLModelManager.getFactory();
	protected String sourceName;
	protected String targetName;
	
	
	/**
	 * @see org.eclipse.gef.commands.Command
	 */			
	public ConnectCommand() {
		super();
	}
	/**
	 * @see org.eclipse.gef.commands.Command
	 */			
	public ConnectCommand(String s) {
		super(s);
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#execute()
	*/
	public void execute() {
			createConnection();
	}
	
	//connecting
	protected void createConnection(){					
		assERS = mfc.createAssociationEndRoleSource();
		assERT = mfc.createAssociationEndRoleTarget();
		
		container.getAssociationRole().add(association);
				
		assERS.setName(sourceName);
		association.setSource(assERS);
		assERS.setAssociationRole(association);
		assERS.setNumberOfAnchor(numberOfSourceAnchor);				
		assERT.setName(targetName);		
		
		association.setTarget(assERT);
		assERT.setAssociationRole(association);
		assERT.setNumberOfAnchor(numberOfTargetAnchor);
				
		source.getSource().add(assERS);
		target.getTarget().add(assERT);
		assERS.setType(source);	
		assERT.setType(target);		
				
	}	
	
	
	/**
	* @see org.eclipse.gef.commands.Command#redo()
	*/
	public void redo() {
		//redo create connection	
		container.getAssociationRole().add(association);		
		association.setSource(assERS);
		assERS.setAssociationRole(association);
		association.setTarget(assERT);
		assERT.setAssociationRole(association);		
		source.getSource().add(assERS);
		target.getTarget().add(assERT);
		assERS.setType(source);	
		assERT.setType(target);		
	}

	/**
	* @see org.eclipse.gef.commands.Command#undo()
	*/	
	public void undo() {
		//delete created connection
		target.getTarget().remove(assERT);
		source.getSource().remove(assERS);
		assERS.setType(null);
		assERT.setType(null);
		association.setSource(null);
		association.setTarget(null);
		container.getAssociationRole().remove(association);			
	}
	

	/**
	 * returns source node
	 * @return source the <code>ClassifierNode</code>
	 */
	public ClassifierNode getSource() {
		return source;
	}
	
	/**
	 * returns association
	 * @return association the <code>AssociationRole</code>
	 */
	public AssociationRole getAssociation(){
		return association;
	}
	
	/**
	 * returns target node
	 * @return target the <code>ClassifierNode</code>
	 */
	public ClassifierNode getTarget() {
		return target;
	}

	/**
	* set source node
	* @param newSource the <code>ClassifierNode</code>
	*/
	public void setSource(ClassifierNode newSource) {
		source = newSource;
	}
	
	/**
	* set target node
	* @param newTarget the <code>ClassifierNode</code>
	*/
	public void setTarget(ClassifierNode newTarget) {
		target = newTarget;
	}
	
	/**
	* set association
	* @param newArc the <code>AssociationRole</code>
	*/
	public void setAssociation(AssociationRole newArc){
		association = newArc;
	}
	
	
	/**
	 * returns container that contains source ,target node etc. 
	 * @return container the <code>NavExpContainer</code>
	 */
	public NavExpContainer getContainer() {
		return container;
	}

	/**
	 * set container that contains source ,target node etc. 
	 * @param container <code>NavExpContainer</code>
	 */
	public void setContainer(NavExpContainer container) {
		this.container = container;
	}

	/**
	 * @return
	 */
	public int getNumberOfSourceAnchor() {
		return numberOfSourceAnchor;
	}

	/**
	 * @return
	 */
	public int getNumberOfTargetAnchor() {
		return numberOfTargetAnchor;
	}

	/**
	 * @param i
	 */
	public void setNumberOfSourceAnchor(int i) {
		numberOfSourceAnchor = i;
	}

	/**
	 * @param i
	 */
	public void setNumberOfTargetAnchor(int i) {
		numberOfTargetAnchor = i;
	}
	/**
	 * @return
	 */
	public String getSourceName() {
		return sourceName;
	}

	/**
	 * @return
	 */
	public String getTargetName() {
		return targetName;
	}

	/**
	 * @param string
	 */
	public void setSourceName(String string) {
		sourceName = string;
	}

	/**
	 * @param string
	 */
	public void setTargetName(String string) {
		targetName = string;
	}

}