/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.connect;

import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.PredicateContainer;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLExpression;
import VOCLModel.VariableDeclaration;

/**
 * Class       :IsInConnectCommand<br/>
 * Package     : vocleditor.commands.connect<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Command to connect two ClassifierRoles with AssociationRole
 * <br/>
 * @author vilapower
 */
public class IsInConnectCommand extends ConnectCommand {
	
	private Constraint constraint;
	private VariableDeclaration var;
	private static final String ConnectionCommand_Label = "associationIsIn";
	private static final String ConnectionCommand_Description = "AssociationRole IsIn connection command";

	/**
	* @see org.eclipse.gef.commands.Command#canExecute()
	*/	
	public boolean canExecute() {
		if(source!=null & target!=null & association!=null){					
			if((source instanceof SetClassifierRole & target instanceof ClassifierRole)||
			   (source instanceof ClassifierRole & target instanceof SetClassifierRole)){
				if(source instanceof SetClassifierRole & target instanceof ClassifierRole){
					if(target.getCallVar()==null)return false;
					if(((SetClassifierRole)source).getPredicateContainer()==null)return false;

					 VOCLExpression pred= (VOCLExpression) target.eContainer();
					 while(!(pred instanceof PredicateContainer)& !(pred instanceof Constraint)){
					 	pred=(VOCLExpression) pred.eContainer();	
					 }
					 if(!((SetClassifierRole)source).getPredicateContainer().equals(pred))return false;
					 
					 String nameIt=((SetClassifierRole)source).getPredicateContainer().getIterator().getVarName();
					 String typeIt=((SetClassifierRole)source).getPredicateContainer().getIterator().getType();
					 if( !(nameIt.equals(target.getCallVar().getVarName())) ||
					 	 !(typeIt.equals(target.getCallVar().getType())))return false;
					 
					return true;
				}
				if(source instanceof ClassifierRole & target instanceof SetClassifierRole){
					if(source.getCallVar()==null)return false;
					if(((SetClassifierRole)target).getPredicateContainer()==null)return false;
					
					 VOCLExpression pred= (VOCLExpression) source.eContainer();
					 while(!(pred instanceof PredicateContainer)& !(pred instanceof Constraint)){
					 	pred=(VOCLExpression) pred.eContainer();	
					 }
					 if(!((SetClassifierRole)target).getPredicateContainer().equals(pred))return false;
					 
					 String nameIt=((SetClassifierRole)target).getPredicateContainer().getIterator().getVarName();
					 String typeIt=((SetClassifierRole)target).getPredicateContainer().getIterator().getType();
					 if( !(nameIt.equals(source.getCallVar().getVarName())) ||
					 	 !(typeIt.equals(source.getCallVar().getType())))return false;
					 
					return true;
				}
			}
		}
		return false;
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		constraint=null;
		VOCLExpression exp=(VOCLExpression)container;
		while(constraint==null){
			if(exp.eContainer() instanceof Constraint)
				constraint=(Constraint) exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}	
		super.execute();
	}
	
	//connecting
	protected void createConnection(){	
		if(target instanceof ClassifierRole){
			var=target.getCallVar();
			constraint.getVariableDeclaration().remove(var);
			target.setCallVar(((SetClassifierRole)source).getPredicateContainer().getIterator());
			targetName="IsIn";
			sourceName="";
		}else{
			var=source.getCallVar();
			constraint.getVariableDeclaration().remove(var);
			source.setCallVar(((SetClassifierRole)target).getPredicateContainer().getIterator());
			sourceName="IsIn";
			targetName="";
		}
		super.createConnection();	
				
	}
	
	
	/**
	* @see org.eclipse.gef.commands.Command#redo()
	*/
	public void redo() {
		constraint.getVariableDeclaration().remove(var);
		if(target instanceof ClassifierRole)
			target.setCallVar(((SetClassifierRole)source).getPredicateContainer().getIterator());
		else
			source.setCallVar(((SetClassifierRole)target).getPredicateContainer().getIterator());
		
		super.redo();
	}

	/**
	* @see org.eclipse.gef.commands.Command#undo()
	*/	
	public void undo() {
		constraint.getVariableDeclaration().add(var);
		if(target instanceof ClassifierRole)
			target.setCallVar(var);
		else
			source.setCallVar(var);
		
		super.undo();
	}
	


}