/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.connect;

import org.eclipse.gef.commands.Command;
import VOCLModel.AssociationEndRole;
import VOCLModel.ClassifierNode;

/**
 * Class       :ReConnectCommand<br/>
 * Package     : vocleditor.commands.connect<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Command to reconnect association
 * <br/>
 * @author vilapower
 */
public class ReConnectCommand extends Command {
	
	private int numberOfSourceAnchor;
	private int numberOfTargetAnchor;
		
	private static final String ConnectionCommand_Label = "associationRole";
	private static final String ConnectionCommand_Description = "AssociationRole connection command";
	private ClassifierNode node;
	private AssociationEndRole assER ;
	private int oldAnchorId = 0;
	private int newAnchorId = 0;	
	
	/**
	 * @see org.eclipse.gef.commands.Command
	 */	
	public ReConnectCommand() {
		super(ConnectionCommand_Label);
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#canExecute()
	*/
	public boolean canExecute() {
		if (assER != null) 
			if (node == assER.getType())return true;
		return false;
	}

	/**
	* @see org.eclipse.gef.commands.Command#execute()
	*/
	public void execute() {
		oldAnchorId = assER.getNumberOfAnchor();
		if (newAnchorId == oldAnchorId)
			return;
		assER.setNumberOfAnchor(newAnchorId);
	}
		
	/**
	* @see org.eclipse.gef.commands.Command#getLabel()
	*/
	public String getLabel() {
		return ConnectionCommand_Description;
	}

	/**
	 * returns the ClassifierNode where the anchornumber will be change
	 * @return node
	 */
	public ClassifierNode getNode() {
		return node;
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#redo()
	*/
	public void redo() {
		//redo create connection
		execute();	
			
	}
	
	/**
	 * set the ClassifierNode where the anchornumber will be change
	 * @param node ClassifierNode
	 */
	public void setNode(ClassifierNode node) {
		this.node = node;
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#undo()
	*/
	public void undo() {
		//delete created connection
		assER.setNumberOfAnchor(oldAnchorId);	
	}
	
	
	/**
	 * returns the AssociationEndRole where the anchornumber will be change
	 * @return node
	 */
	public AssociationEndRole getAssER() {
		return assER;
	}


	
	/**
	 * set the AssociationEndRole where the anchornumber will be change
	 * @param source
	 */
	public void setAssERS(AssociationEndRole endR) {
		assER = endR;
	}
	

	/**
	 * returns new anchornumber
	 * @return newAnchorId as Integer
	 */
	public int getNumberOfAnchor() {
		return newAnchorId;
	}

	/**
	 * set new anchornumber 
	 * @param i
	 */
	public void setNumberOfAnchor(int i) {
		newAnchorId = i;
	}



}