/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.constraints;


import org.eclipse.gef.commands.Command;

import VOCLModel.BoolExp;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.VOCLExpression;


public class MoveSashCommand extends Command
{


	VOCLExpression exp = null;
	double proportion = 0;
	double oldprop	  = 0;
	private boolean vertical;

	/**
	 * Constructor of this command
	 * @param exp - the expression where the sash is nested in
	 * @param proportion - a double value representing the proportion of the parts
	 */
	public MoveSashCommand(VOCLExpression exp, double proportion){
		this.exp = exp;
		this.proportion = proportion;
	}
	
	/**
	 * Constructor of this command
	 * @param exp - the expression where the sash is nested in
	 * @param proportion - a double value representing the proportion of the parts
	 * @param v - true, if the expression is oriented vertically
	 **/
	public MoveSashCommand(VOCLExpression exp, double proportion,boolean v){
			this.exp = exp;
			this.proportion = proportion;
			vertical=v;
	}

	/**
	 * The execution of the moving is implemented here.
	 */
	public void execute(){		
		if(exp instanceof BoolExp) {
			oldprop = ((BoolExp)exp).getProportion();
			((BoolExp)exp).setProportion(proportion);
		}
		if(exp instanceof LetExp) {
			oldprop = ((LetExp)exp).getProportion();
			((LetExp)exp).setProportion(proportion);
		}
		if(exp instanceof IfExp){
			if(vertical) {
				oldprop = ((IfExp)exp).getProportionV();
				((IfExp)exp).setProportionV(proportion);
			} 
			else {
				oldprop = ((IfExp)exp).getProportionH();
				((IfExp)exp).setProportionH(proportion);
			} 
		}
			
	}

	/**
	 * Redo of the undo of a moved sash.
	 */
	public void redo()
	{
		if(exp instanceof BoolExp) {
			((BoolExp)exp).setProportion(proportion);
		}
		if(exp instanceof LetExp) {
			((LetExp)exp).setProportion(proportion);
		}
		if(exp instanceof IfExp){
			if(vertical) {
				((IfExp)exp).setProportionV(proportion);
			} 
			else {
				((IfExp)exp).setProportionH(proportion);
			} 
		}
	}

	/**
	 * undos the sash-moving 
	 */
	public void undo() {
		if(exp instanceof BoolExp) {
			((BoolExp)exp).setProportion(oldprop);
		}
		if(exp instanceof LetExp) {
			((LetExp)exp).setProportion(oldprop);
		}
		if(exp instanceof IfExp){
			if(vertical) {
				((IfExp)exp).setProportionV(oldprop);
			} 
			else {
				((IfExp)exp).setProportionH(oldprop);
			} 
		}	
	}
}
