/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.constraints;


import org.eclipse.gef.EditPart;
import org.eclipse.gef.tools.DragEditPartsTracker;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;



public class SashTracker extends DragEditPartsTracker
{

	EditPart part;

	/**
	 * Constructor of this class
	 * @param source - the source-editpart
	 * @param part - tartget-editpart
	 */
	public SashTracker(EditPart source, EditPart part){
		super(source);
		this.part = part;
	}
	
	/**
	 * Getter for the target-editpart
	 * @return the target editpart
	 */
	protected EditPart getTargetEditPart(){
		return part;
	}

	/**
	 * Checks if the sash in in drag-mode
	 * @return - true, if the sash is dragged
	 */
	boolean isInDragInProgressp() {
		return isInState(STATE_DRAG_IN_PROGRESS | STATE_ACCESSIBLE_DRAG_IN_PROGRESS);
	}
	
	/**
	 * The execution of the drag of the sash
	 * @return true, if dagging was successful
	 */
	protected boolean handleDragInProgress() {
		if (isInDragInProgressp()) {
			updateTargetRequest();
			updateTargetUnderMouse();
			setCurrentCommand(getCommand());
		}
		return true;
	}

	/**
	 * This method calculates the cursor - position
	 * @return the actual cursor, needed for dragging
	 */
	protected Cursor calculateCursor() {	
		if (isInState(STATE_INITIAL | STATE_DRAG | STATE_ACCESSIBLE_DRAG | STATE_DRAG_IN_PROGRESS))
			return new Cursor(null,SWT.CURSOR_SIZEALL);
		if (isInState( STATE_ACCESSIBLE_DRAG_IN_PROGRESS | STATE_DRAG_IN_PROGRESS))
					return new Cursor(null,SWT.CURSOR_CROSS);
			
		return super.calculateCursor();
	}

}
