/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.constraints;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;


import VOCLModel.ClassifierNode;


public class SetConstraintClassifierRoleCommand
	extends org.eclipse.gef.commands.Command
{
	private static final String Command_Label_Location = "change location command";
	private static final String Command_Label_Resize = "resize command";
	private Point newPos;
	
	private Point oldPos;
	
	private ClassifierNode part;
	
	/**
	 * Execution of this command is for resizing and positioning
	 * of the Constraint
	 */
	public void execute() {
		oldPos  = new Point( part.getX(), part.getY() );
		part.setX(newPos.x);
		part.setY(newPos.y);
	}
	
	/**
	 * Getter for the label of this command
	 * @return the string value of this command-label
	 */
	public String getLabel(){
		return Command_Label_Resize;
	}
	
	/**
	 * Redos an undo of this command.
	 */
	public void redo() {
		part.setX(newPos.x);
		part.setY(newPos.y);

	}
	
	/**
	 * Setter for the new location of the classifier
	 * @param r - a rectangle holding the new location
	 */
	public void setLocation(Rectangle r){
		setLocation(r.getLocation());
	}
	
	/**
	 * Setter for the new location of the classifier.
	 * @param p - a point holding the new location
	 */
	public void setLocation(Point p) {
		newPos = p;
	}
	
	/**
	 * Setter of the ClassifierNode
	 * @param part - the responsible classifiernode
	 */
	public void setPart(ClassifierNode part) {
		this.part = part;
	}
	
	/**
	 * Invalidates a change made by this command
	 */	
	public void undo() {
		part.setX(oldPos.x);
		part.setY(oldPos.y);

	}
	
}
