/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.create;

import vocleditor.model.VOCLModelManager;
import VOCLModel.ExpressionType;
import VOCLModel.LetExp;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VOCLSash;
import VOCLModel.VariableDeclaration;

/**
 * @author eddi
 */
public class CreateLetExpCommand extends CreateExpressionCommand {

	protected VOCLSash sash;
	private int kind=0;
	private boolean vertical=true;
	private String varname="";
	private String vartype="";
	
	VariableDeclaration var;
	
	/**
	 * Execution of this command creates a new LetExpression.
	 */
	public void execute() {
		VOCLModel4Factory mfc = VOCLModelManager.getFactory();
		
		sash=mfc.createVOCLSash();
		sash.setVertikal(isVertical());
		((LetExp)expression).setVOCLSash(sash);
		((LetExp)expression).setProportion(0.5);
		
		var=mfc.createVariableDeclaration();
		var.setVarName(varname);
		var.setType(vartype);
		((LetExp)expression).setVariableDeclaration(var);
		constraint.getVariableDeclaration().add(var);
		NavExpContainer container1=mfc.createNavExpContainer();
		dublicate.dublicateContainer(container1,navexp);
		container1.setExptype(ExpressionType.LET_E_LITERAL);
		((LetExp)expression).setLetExpression(container1);
	
		NavExpContainer container2=mfc.createNavExpContainer();
		dublicate.dublicateContainer(container2,navexp);
		container2.setExptype(ExpressionType.IN_E_LITERAL);
		((LetExp)expression).setInExpression(container2);

		super.execute();
	}
	
	/**
	 * Retrieves the kind of this Expression as ineteger
	 * @return - the kind
	 */
	public int getKind() {
		return kind;
	}

	/**
	 * Checks if the layout of this expression is vertical or not
	 * @return- true if vertical is selected
	 */
	public boolean isVertical() {
		return vertical;
	}

	/**
	 * Sets the kind of this expresiion
	 * @param string
	 */
	public void setKind(int i) {
		kind = i;
	}

	/**
	 * Sets the layout of this expression
	 * @param b - if b is true, the layout is vertical, 
	 * else horizontal
	 */
	public void setVertical(boolean b) {
		vertical = b;
	}



	/** 
	 * Undos the action done by this command
	 */
	public void undo(){
		((LetExp)expression).setVariableDeclaration(null);
		constraint.getVariableDeclaration().remove(var);
		super.undo();
	}
	
	@Override
	public void redo() {
		constraint.getVariableDeclaration().add(var);
		((LetExp)expression).setVariableDeclaration(var);
		super.redo();
	}

	/**
	 * Retrieves the varName for this expression
	 * @return - the variables name as string-value
	 */
	public String getVarname() {
		return varname;
	}

	/**
	 * Retrieves the variables type of the let-expression.
	 * @return - the type of the variable
	 */
	public String getVartype() {
		return vartype;
	}

	/**
	 * Sets the variables name of the let-expression
	 * @param string - the new name
	 */
	public void setVarname(String string) {
		varname = string;
	}

	/**
	 * Sets the variables type of the let-expression
	 * @param string - the new type
	 */
	public void setVartype(String string) {
		vartype = string;
	}

}
