/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
 
package vocleditor.commands.delete;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import VOCLModel.Attribute;
import VOCLModel.ClassifierNode;
import VOCLModel.Constraint;
import VOCLModel.NavExpContainer;
import VOCLModel.VariableDeclaration;

/**
 * @author eddi
 */
public abstract class DeleteClassifierNode{

	protected VariableDeclaration var;
	protected ClassifierNode delClassifier;
	protected NavExpContainer parent;
	protected Constraint constraint;
	protected List variablen= new LinkedList();
	protected DeleteAssociationAtClassifierNode deleteAssociation;



	/**
	 * 
	 */
	public DeleteClassifierNode(Constraint constraint,VariableDeclaration var,NavExpContainer parent,ClassifierNode delClassifier) {
		this.var=var;
		this.constraint=constraint;
		this.parent=parent;
		this.delClassifier=delClassifier;
		deleteAssociation=new DeleteAssociationAtClassifierNode(delClassifier);
	}
	/**
	 * Execution of this command deletes a classifierRole
	 * completely.
	 */
	public void execute() {
		variablen.clear();
		deleteAssociation.execute();
		deleteVariable();
		deleteAttrbute();
		deleteNode();
		delete();
	}
	
	
	private void deleteAttrbute(){
		Iterator i=delClassifier.getAttribute().iterator();
		while(i.hasNext()){
			Attribute att=(Attribute) i.next();
			if(att.getVariableDeclaration()!=null){
				variablen.add(att.getVariableDeclaration());
				constraint.getVariableDeclaration().remove(att.getVariableDeclaration());	
			}	
		}
	}

	/**
	 * Undos an action done by this command
	 */
	public void undo(){
		addVariable();
		addNode();
		constraint.getVariableDeclaration().addAll(variablen);
		deleteAssociation.undo();
	}
	
	public abstract void addVariable();
	
	public abstract void deleteVariable();
	
	public abstract void deleteNode();
	
	public abstract void addNode();
	
	public abstract void delete();
	
}
