/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
 
package vocleditor.commands.delete;

import org.eclipse.gef.commands.Command;

import VOCLModel.ClassifierNode;
import VOCLModel.Constraint;
import VOCLModel.NavExpContainer;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLExpression;
import VOCLModel.VariableDeclaration;

/**
 * @author eddi
 */
public class DeleteClassifierNodeCommand extends Command {

	private static final String DeleteCommand_Label = "DeletePlaceCommand";
	private	VariableDeclaration var;
	private	ClassifierNode delClassifier;
	private NavExpContainer parent;
	private Constraint constraint;
	private DeleteSetClassifierRole deleteSetClassifier;
	private DeleteClassifierRole deleteClassifier;

	/**
	 * Constructor of this command - no parameters needed
	 */
	public DeleteClassifierNodeCommand() {
		super( DeleteCommand_Label );
	}
	
	/**
	 * Execution of this command deletes a classifierRole
	 * completely.
	 */
	public void execute() {
		if(delClassifier instanceof SetClassifierRole){
			deleteSetClassifier=new DeleteSetClassifierRole(constraint,
															var,
															parent,
															delClassifier);
			deleteSetClassifier.execute();
		}else{
			deleteClassifier=new DeleteClassifierRole(constraint,
															var,
															parent,
															delClassifier);
			deleteClassifier.execute();
		}
	}

	/**
	 * Redos an undo.
	 */
	public void redo() {
		if(delClassifier instanceof SetClassifierRole){
			deleteSetClassifier.execute();
		}else{
			deleteClassifier.execute();
		}
	}
	/**
	 * Undos an action done by this command
	 */
	public void undo() {
		if(delClassifier instanceof SetClassifierRole){
			deleteSetClassifier.undo();
		}else{
			deleteClassifier.undo();
		}
	}
	
	/**
	 * Setter for the classifierNode
	 * @param cls - the new node
	 */
	public void setClassifierNode(ClassifierNode cls) {
		this.delClassifier = cls;
		var=cls.getCallVar();
	}

	/**
	 * Setter for the container, where the node is placed in
	 * @param container
	 */
	public void setParent(NavExpContainer container) {
		this.parent = container;
		constraint=null;
		VOCLExpression exp=parent;
		while(constraint==null){
			if(exp.eContainer() instanceof Constraint)constraint=exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}
	}
		
	
}
