/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.delete;

import org.eclipse.gef.commands.Command;

import VOCLModel.Constraint;
import VOCLModel.VOCLDiagramm;


/**
 * @author eddi
 */
public class DeleteConstraintCommand extends Command {

	private Constraint constraint;
	private VOCLDiagramm diagramm;

	/**
	 * Ths constructor of this command
	 */
	public DeleteConstraintCommand() {
		super();
	}

	/**
	 * The constructor of this command with a specified label
	 * @param label - the label indicating this command
	 */
	public DeleteConstraintCommand(String label) {
		super(label);
	}

	/**
	 * The execution of this command deletes a complete 
	 * VOCLExpression
	 */
	public void execute() {
		diagramm.getConstraint().remove(constraint);
	}
	
	
	/**
	 * Redos a deletion of a VOCLExpression
	 */
	public void redo() {
		diagramm.getConstraint().remove(constraint);
	}

	/**
	 * Undos the deletion of a VOCLExpression
	 */
	public void undo() {
		diagramm.getConstraint().add(constraint);
	}


	/**
	 * Expression to delete
	 * @param expression
	 */
	public void setConstraint(Constraint constraint) {
		this.constraint = constraint;
	}

	

	/**
	 * Container,where the expression will be delete 
	 * @param container
	 */
	public void setVOCLDiagramm(VOCLDiagramm diagramm) {
		this.diagramm = diagramm;
	}


}
