/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;





import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;

import vocleditor.action.request.ClassifierRolePropertiesRequest;
import vocleditor.commands.adds.ClassifierPropsCommand;
import vocleditor.figure.ClassifierRoleFigure;
import VOCLModel.ClassifierRole;

/**
 * Class       :SimpleClassifierNodeEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for SimpleClassifierRole
 * <br/>
 * @author vilapower
 */
public class ClassifierRoleEditPart 
	extends AbstractClassifierNodeEditPart{
	
    /**
     * Creates a new SimpleClassifierRoleEditPart instance.
     * @param element SimpleClassifierRole
     */
    public ClassifierRoleEditPart(ClassifierRole clr){
		setModel(clr);
    }

	/** 
	 * This method instanciates and returns a SimpleClassifierRoleFigure
	 * with class-name, variable-name, and a list of attributes. 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {	
		// retrieving class data
		ClassifierRole clr =(ClassifierRole)this.getClassifierNode();
		String classname = clr.getType();
		String varname="";
		if(clr.getCallVar()!=null){
			varname   = clr.getCallVar().getVarName();		
		}
		// create a new instance of a ClassifierRoleFigure
		ClassifierRoleFigure figure = 
						new ClassifierRoleFigure(varname,classname);
						
		if(clr.getMethod()!=null){
			figure.setMethod(true);
			figure.setMethodStr(clr.getMethod());
		}else figure.setMethod(false);	
				
		// retrieve the attributes and add them to the figure		
		figure.setAttributes(clr.getAttribute());
		
		// finally return the figure
		return figure;
	}
	

	/**
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		if (req instanceof ClassifierRolePropertiesRequest) return true;
		return super.understandsRequest(req);
	}

	/**
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {
		if (request instanceof ClassifierRolePropertiesRequest)
			return new ClassifierPropsCommand();
			
		return super.getCommand(request);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		ClassifierRoleFigure figure=(ClassifierRoleFigure) getNodeFigure();
		if(((ClassifierRole)getClassifierNode()).getMethod()==null){
			figure.setMethod(false);	
		}else{
			figure.setMethod(true);
			figure.setMethodStr(((ClassifierRole)getClassifierNode()).getMethod());
		}
		figure.setAttributes(getClassifierNode().getAttribute());
		String name="";
		if(getClassifierNode().getCallVar()!=null)
			name=getClassifierNode().getCallVar().getVarName();
		figure.setClassVarName(name);
		super.refreshVisuals();
	}

}
