/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;

import vocleditor.edit.policy.IfExpXYLayoutEditPolicy;
import vocleditor.edit.policy.VOCLEditPolicy;
import vocleditor.figure.IfExpFigure;
import VOCLModel.ExpressionType;
import VOCLModel.IfExp;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Package;
import VOCLModel.VOCLSash;

/**
 * Class       :IfExpEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for IfExp
 * <br/>
 * @author vilapower
 */
public class IfExpEditPart extends AbstractExpressionEditPart {



	/**
	 * Creates a new IfExpEditPart instance.
	 * @param element VOCLExpression as IfExp
	 */
	public IfExpEditPart(VOCLExpression container) {
		setModel(container);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONTAINER_ROLE,  new VOCLEditPolicy());	
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new IfExpXYLayoutEditPolicy());	
		super.createEditPolicies();	
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		return new IfExpFigure();
	}

	/**
	 * returns model of this EditPart
	 * @return IfExp
	 */	
	protected  IfExp getIfExp(){
		return (IfExp) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		List	allChildren = new LinkedList();	
		if(getIfExp().getIfExpression()!=null)
			allChildren.add(getIfExp().getIfExpression());
		if(getIfExp().getThenExpression()!=null)
			allChildren.add(getIfExp().getThenExpression());
		if(getIfExp().getElseExpression()!=null)
			allChildren.add(getIfExp().getElseExpression());
		if(getIfExp().getHorizontal()!=null)
			allChildren.add(getIfExp().getHorizontal());
		if(getIfExp().getVertikal()!=null)
			allChildren.add(getIfExp().getVertikal());
		return allChildren;
	}
	
	
	/**
	 * checks if the model was be changed
	* @param notification org.eclipse.emf.common.notify.Notification
	*/		
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		int featureId = notification.getFeatureID(VOCLModel4Package.class);
		switch(featureId){

		case VOCLModel4Package.IF_EXP__PROPORTION_H:
		case VOCLModel4Package.IF_EXP__PROPORTION_V:
		case VOCLModel4Package.IF_EXP__EXPTYPE:
			refreshVisuals();
			break;
		case VOCLModel4Package.IF_EXP__IF_EXPRESSION:	
		case VOCLModel4Package.IF_EXP__THEN_EXPRESSION:	
		case VOCLModel4Package.IF_EXP__ELSE_EXPRESSION:	
		case VOCLModel4Package.IF_EXP__HORIZONTAL:
		case VOCLModel4Package.IF_EXP__VERTIKAL:	
			refreshVisuals();
			break;
		default:
			refreshVisuals();
		}
		
	}

	/**
	 * returns Figure of this EditPart
	 * @return IfExpFigure
	 */
	protected IfExpFigure getOwnFigure() {
		return (IfExpFigure)getFigure();
	}

	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getExpression()
	 */
	public VOCLExpression getExpression() {
		return getIfExp();
	}

	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getPrivateContentPane(org.eclipse.gef.EditPart)
	 */
	public IFigure getPrivateContentPane(EditPart child) {
		if(child instanceof VOCLSashEditPart){
			if(getIfExp().getHorizontal()==child.getModel()){
				return getOwnFigure().getHPanel();
			}else{
				return getOwnFigure().getVPanel();
			}
		}else{
			VOCLExpression model=((AbstractExpressionEditPart)child).getExpression();
			if(model.getExptype()==ExpressionType.get(3))
				return getOwnFigure().getIfpanel();
			else if(model.getExptype()==ExpressionType.get(4))
				return getOwnFigure().getThenpanel();
			else return getOwnFigure().getElsepanel();
		}
	}
	
	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getPrivateContentPane(java.lang.Object)
	 */
	public IFigure getPrivateContentPane(Object child) {
		if(child instanceof VOCLSash){
			if(getIfExp().getHorizontal()==child){
				return getOwnFigure().getHPanel();
			}else{
				return getOwnFigure().getVPanel();
			}
		}else{
			VOCLExpression model=(VOCLExpression)child;
			if(model.getExptype()==ExpressionType.get(3))
				return getOwnFigure().getIfpanel();
			else if(model.getExptype()==ExpressionType.get(4))
				return getOwnFigure().getThenpanel();
			else return getOwnFigure().getElsepanel();
		}
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshChildren()
	 */
	protected void refreshChildren() {		
		int i;
		EditPart editPart;

		Map modelOfSashHToEditPart = new HashMap();
		Map modelOfSashVToEditPart = new HashMap();
		Map modelOfIfToEditPart = new HashMap();
		Map modelOfThenToEditPart = new HashMap();
		Map modelOfElseToEditPart = new HashMap();
		
		List children = getChildren();
		
		List childrenAsSashH=new LinkedList();
		List childrenAsSashV=new LinkedList();
		List childrenAsIf=new LinkedList();
		List childrenAsThen=new LinkedList();
		List childrenAsElse=new LinkedList();
		
		for (i = 0; i < children.size(); i++) {
			editPart = (EditPart)children.get(i);
			if(editPart instanceof VOCLSashEditPart){	
				if(getIfExp().getHorizontal()==editPart.getModel()){
					modelOfSashHToEditPart.put(editPart.getModel(), editPart);
					childrenAsSashH.add(editPart);
				}else{
					modelOfSashVToEditPart.put(editPart.getModel(), editPart);
					childrenAsSashV.add(editPart);
				}
			}else{
				VOCLExpression model=((AbstractExpressionEditPart)editPart).getExpression();
				if(model.getExptype()==ExpressionType.get(3)){
					modelOfIfToEditPart.put(editPart.getModel(), editPart);
					childrenAsIf.add(editPart);	
				}else if(model.getExptype()==ExpressionType.get(4)){
					modelOfThenToEditPart.put(editPart.getModel(), editPart);
					childrenAsThen.add(editPart);
				 }else{
					modelOfElseToEditPart.put(editPart.getModel(), editPart);
					childrenAsElse.add(editPart); 	
				 }
			}
		}
		
		List modelObjectsOfSashH =new LinkedList();
		List modelObjectsOfSashV =new LinkedList();
		List modelObjectsOfIf =new LinkedList();
		List modelObjectsOfThen =new LinkedList();
		List modelObjectsOfElse =new LinkedList();
		
		Iterator it=getModelChildren().iterator();
		while(it.hasNext()){
			Object model=it.next();
			if(model instanceof VOCLSash){	
				if(getIfExp().getHorizontal()==model){
					modelObjectsOfSashH.add(model);
				}else{
					modelObjectsOfSashV.add(model);
				}
			}else{
				VOCLExpression mod=(VOCLExpression)model;	
				if(mod.getExptype()==ExpressionType.get(3)){
					modelObjectsOfIf.add(model);
				}else if(mod.getExptype()==ExpressionType.get(4)){
					modelObjectsOfThen.add(model);	
				}else{
					modelObjectsOfElse.add(model);
				}
			}
		}
		refreshPrivateChildren(modelObjectsOfSashH,modelOfSashHToEditPart,childrenAsSashH);
		refreshPrivateChildren(modelObjectsOfSashV,modelOfSashVToEditPart,childrenAsSashV);
		refreshPrivateChildren(modelObjectsOfIf,modelOfIfToEditPart,childrenAsIf);
		refreshPrivateChildren(modelObjectsOfThen,modelOfThenToEditPart,childrenAsThen);
		refreshPrivateChildren(modelObjectsOfElse,modelOfElseToEditPart,childrenAsElse);
	}
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		getOwnFigure().setProportionH(getIfExp().getProportionH());
		getOwnFigure().setProportionV(getIfExp().getProportionV());
		super.refreshVisuals();
		refreshChildren();
	}

}
