/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;

import vocleditor.edit.policy.LetExpXYLayoutEditPolicy;
import vocleditor.edit.policy.VOCLEditPolicy;
import vocleditor.figure.BoolExpFigure;
import vocleditor.figure.LetExpFigure;
import VOCLModel.ExpressionType;
import VOCLModel.LetExp;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Package;
import VOCLModel.VOCLSash;

/**
 * Class       :LetExpEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for LetExp
 * <br/>
 * @author vilapower
 */
public class LetExpEditPart extends AbstractExpressionEditPart {



	/**
	 * Creates a new LetExpEditPart instance.
	 * @param element VOCLExpression as LetExp
	 */
	public LetExpEditPart(VOCLExpression container) {
		setModel(container);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONTAINER_ROLE,  new VOCLEditPolicy());	
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new LetExpXYLayoutEditPolicy());	
		super.createEditPolicies();	
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		String  var="";
		var=getLetExp().getVariableDeclaration().getVarName()+"[ "+
								getLetExp().getVariableDeclaration().getType()+" ]";
		LetExpFigure figure=
				new LetExpFigure(getLetExp().getVOCLSash().isVertikal()
								 ,"",var);
		return figure;
	}
	
	/**
	 * returns model of this EditPart
	 * @return LetExp
	 */
	protected  LetExp getLetExp(){
		return (LetExp) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		List	allChildren = new LinkedList();	
		if(getLetExp().getInExpression()!=null)
			allChildren.add(getLetExp().getInExpression());
		if(getLetExp().getLetExpression()!=null)
			allChildren.add(getLetExp().getLetExpression());
		if(getLetExp().getVOCLSash()!=null)
			allChildren.add(getLetExp().getVOCLSash());
		return allChildren;
	}
	
	
	/**
	 * checks if the model was be changed
	* @param notification org.eclipse.emf.common.notify.Notification
	*/	
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		int featureId = notification.getFeatureID(VOCLModel4Package.class);
		switch(featureId){

		case VOCLModel4Package.LET_EXP__IS_NEGATED:
		case VOCLModel4Package.LET_EXP__PROPORTION:
		case VOCLModel4Package.LET_EXP__EXPTYPE:
			refreshVisuals();
			break;
		case VOCLModel4Package.LET_EXP__LET_EXPRESSION:	
		case VOCLModel4Package.LET_EXP__IN_EXPRESSION:	
		case VOCLModel4Package.LET_EXP__VOCL_SASH:	
			refreshVisuals();
			break;
		default:
			refreshVisuals();
		}
		
	}

	/**
	 * LetExpFigure extends BoolExpFigure
	 * @see vocleditor.edit.BoolExpEditPart#getOwnFigure()
	 * LetExpFigure extends BoolExpFigure
	 * @see LetExpFigure
	 */
	protected BoolExpFigure getOwnFigure() {
		return (LetExpFigure)getFigure();
	}

	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getExpression()
	 */
	public VOCLExpression getExpression() {
		return getLetExp();
	}

	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getPrivateContentPane(org.eclipse.gef.EditPart)
	 */
	public IFigure getPrivateContentPane(EditPart child) {
		if(child instanceof VOCLSashEditPart)
			return getOwnFigure().getPanel();
		else{
			VOCLExpression model=((AbstractExpressionEditPart)child).getExpression();
			if(model.getExptype()==ExpressionType.get(6))
				return getOwnFigure().getFirstPanel();
			else return getOwnFigure().getSecondPanel();
		}
	}
	
	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getPrivateContentPane(java.lang.Object)
	 */
	public IFigure getPrivateContentPane(Object child) {
		if(child instanceof VOCLSash)
			return getOwnFigure().getPanel();
		else{
			VOCLExpression model=(VOCLExpression)child;
			if(model.getExptype()==ExpressionType.get(6))
				return getOwnFigure().getFirstPanel();
			else return getOwnFigure().getSecondPanel();
		}
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshChildren()
	 */
	protected void refreshChildren() {		
		int i;
		EditPart editPart;

		Map modelOfSashToEditPart = new HashMap();
		Map modelOfLeftToEditPart = new HashMap();
		Map modelOfRightToEditPart = new HashMap();
		
		List children = getChildren();
		
		List childrenAsSash=new LinkedList();
		List childrenAsLeft=new LinkedList();
		List childrenAsRight=new LinkedList();
		
		for (i = 0; i < children.size(); i++) {
			editPart = (EditPart)children.get(i);
			if(editPart instanceof VOCLSashEditPart){
				modelOfSashToEditPart.put(editPart.getModel(), editPart);
				childrenAsSash.add(editPart);
			}else{
				VOCLExpression model=((AbstractExpressionEditPart)editPart).getExpression();
				if(model.getExptype()==ExpressionType.get(6)){
					modelOfLeftToEditPart.put(editPart.getModel(), editPart);
					childrenAsLeft.add(editPart);	
				}else{
					modelOfRightToEditPart.put(editPart.getModel(), editPart);
					childrenAsRight.add(editPart);
				 }
			}
		}
		
		List modelObjectsOfSash =new LinkedList();
		List modelObjectsOfLeft =new LinkedList();
		List modelObjectsOfRight =new LinkedList();
		
		Iterator it=getModelChildren().iterator();
		while(it.hasNext()){
			Object model=it.next();
			if(model instanceof VOCLSash)
				modelObjectsOfSash.add(model);
			else{
				VOCLExpression mod=(VOCLExpression)model;
				if(mod.getExptype()==ExpressionType.get(6)){
					modelObjectsOfLeft.add(model);	
				}else{
					modelObjectsOfRight.add(model);	
				}	
				
			}
		}
		refreshPrivateChildren(modelObjectsOfSash,modelOfSashToEditPart,childrenAsSash);
		refreshPrivateChildren(modelObjectsOfLeft,modelOfLeftToEditPart,childrenAsLeft);
		refreshPrivateChildren(modelObjectsOfRight,modelOfRightToEditPart,childrenAsRight);
	}
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		getOwnFigure().setProportion(getLetExp().getProportion());
		super.refreshVisuals();
		refreshChildren();
	}

}
