/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;

import vocleditor.action.request.SetClassifierRolePropertiesRequest;
import vocleditor.commands.adds.SetClassifierPropsCommand;
import vocleditor.figure.SetClassifierRoleFigure;
import VOCLModel.SetClassifierRole;

/**
 * Class       :CollClassifierNodeEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for CollClassifierRole
 * <br/>
 * @author vilapower
 */
public class SetClassifierRoleEditPart 
	extends AbstractClassifierNodeEditPart{

	//private Shell shell;
	
    /**
     * Creates a new CollClassifierRoleEditPart instance.
     * @param element CollClassifierRole
     */
    public SetClassifierRoleEditPart(SetClassifierRole clr){
		setModel(clr);
    }

	/** 
	 * This method instanciates and returns a CollClassifierRoleFigure
	 * with class-name, variable-name, and a list of attributes. 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		// retrieving class data
		SetClassifierRole clr =(SetClassifierRole)this.getClassifierNode();
		String classname = clr.getType();
		String varname="";
		if(clr.getCallVar()!=null){
			varname   = clr.getCallVar().getVarName();		
		}	
		// create a new instance of a ClassifierRoleFigure
		SetClassifierRoleFigure figure =
					 new SetClassifierRoleFigure(varname,classname, clr);
		return figure;
	}
		
	/**
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		if (req instanceof SetClassifierRolePropertiesRequest) return true;
		boolean b = super.understandsRequest(req);
		return b;
	}

	/**
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {
		if (request instanceof SetClassifierRolePropertiesRequest) 
		   return new SetClassifierPropsCommand();
		return super.getCommand(request);
	}

}
