/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit.policy;


import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionRectangle;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ChangeBoundsRequest;

import vocleditor.commands.constraints.MoveSashCommand;
import vocleditor.edit.VOCLSashEditPart;
import vocleditor.figure.VOCLBorderLayout;
import VOCLModel.BoolExp;
import VOCLModel.VOCLSash;






public class BoolExpXYLayoutEditPolicy extends VOCLExpressionXYLayoutEditPolicy {
	
	private double proportion;

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		if(child instanceof VOCLSashEditPart)
			return new MoveSashCommand((BoolExp)getHost().getModel(),proportion);
		return null;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getConstraintFor(org.eclipse.gef.requests.ChangeBoundsRequest, org.eclipse.gef.GraphicalEditPart)
	 */
	protected Object getConstraintFor(ChangeBoundsRequest request,GraphicalEditPart child) {
		if(child instanceof VOCLSashEditPart){
			IFigure container = child.getFigure().getParent();
			Rectangle p=container.getClientArea();
			Point st=container.getClientArea().getLocation();
			container.translateToAbsolute(st);
			Rectangle rect = new PrecisionRectangle(child.getFigure().getBounds());
			child.getFigure().translateToAbsolute(rect);
			Rectangle original = rect.getCopy();
			rect = request.getTransformedRectangle(rect);	
			VOCLSash sash=(VOCLSash)((VOCLSashEditPart) child).getModel();
			
			if(sash.isVertikal())proportion=((double)(rect.x - st.x))/(p.width);
			else proportion=((double)(rect.y - st.y))/(p.height);
			
			if(proportion<0.01)proportion=0.05;		
			if(proportion>1)proportion=0.95;	
			
			return VOCLBorderLayout.CENTER;
		}	
		return BorderLayout.CENTER;
	}


}
