/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit.policy;


import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionRectangle;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import vocleditor.commands.constraints.SetConstraintClassifierRoleCommand;
import vocleditor.commands.constraints.SetConstraintForPredicatContainerCommand;
import vocleditor.commands.create.CreateBoolExpCommand;
import vocleditor.commands.create.CreateClassifierNodeCommand;
import vocleditor.commands.create.CreateIfExpCommand;
import vocleditor.commands.create.CreateLetExpCommand;
import vocleditor.commands.create.CreatePredicatContainerCommand;
import vocleditor.edit.AbstractClassifierNodeEditPart;
import vocleditor.edit.ConditionEditPart;
import vocleditor.edit.NavExpContainerEditPart;
import vocleditor.edit.PredicatContainerEditPart;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierNode;
import VOCLModel.SetClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.NavExpContainer;
import VOCLModel.PredicateContainer;
import VOCLModel.ClassifierRole;
import VOCLModel.VOCLExpression;





public class NavExpContainerXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		Object	newObjectType = request.getNewObjectType();
		Object	newObject = request.getNewObject();
		Command	createCommand = null;
		NavExpContainer navexp=(NavExpContainer)getHost().getModel();
		if( newObjectType == ClassifierRole.class ||
					newObjectType == SetClassifierRole.class){
			CreateClassifierNodeCommand create = 
					  new CreateClassifierNodeCommand();
			create.setClassiefierNode((ClassifierNode)newObject);
			create.setParent(navexp);
			Rectangle constraint = (Rectangle)getConstraintFor(request);
			Constraint con=null;
			VOCLExpression exp=(VOCLExpression)navexp;
			while(con==null){
				if(exp.eContainer() instanceof Constraint)con=(Constraint) exp.getConstraint();
				else exp=(VOCLExpression) exp.eContainer();	
			}
			create.setConstraint(con);
			create.setLocation(constraint);
			create.setLabel("create a classifiernode");
			createCommand = create;
		}else if( newObjectType == BoolExp.class){
			CreateBoolExpCommand create = new CreateBoolExpCommand();
			create.setName("Bool");
			create.setExpression((VOCLExpression)newObject);
			create.setContainer((VOCLExpression)navexp.eContainer());
			create.setNavexp(navexp);
			createCommand = create;
		}else if( newObjectType == LetExp.class){
			CreateLetExpCommand create = new CreateLetExpCommand();
			create.setName("Let");
			create.setExpression((VOCLExpression)newObject);
			create.setContainer((VOCLExpression)navexp.eContainer());
			create.setNavexp(navexp);
			createCommand = create;
		}else if( newObjectType == IfExp.class){
			CreateIfExpCommand create = new CreateIfExpCommand();
			create.setName("If");
			create.setExpression((VOCLExpression)newObject);
			create.setContainer((VOCLExpression)navexp.eContainer());
			create.setNavexp(navexp);
			createCommand = create;
		}else if( newObjectType == PredicateContainer.class){
			CreatePredicatContainerCommand create = new CreatePredicatContainerCommand();
			create.setSelectionContainer((PredicateContainer)newObject);
			create.setNavcontainer(navexp);
			Rectangle constraint = (Rectangle)getConstraintFor(request);
			create.setLocation(constraint);
			createCommand = create;
		}
		return createCommand;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	protected EditPolicy createChildEditPolicy(EditPart child) {
		if(child instanceof PredicatContainerEditPart)
			return super.createChildEditPolicy(child);
		return new NonResizableEditPolicy();
	}
	




	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		if(child instanceof AbstractClassifierNodeEditPart){ 
			SetConstraintClassifierRoleCommand locationCommand =
									new SetConstraintClassifierRoleCommand();
			locationCommand.setPart((ClassifierNode)child.getModel());		
			locationCommand.setLocation((Rectangle)constraint);
			return locationCommand;
		}else if(child instanceof PredicatContainerEditPart){ 
			SetConstraintForPredicatContainerCommand command =
									new SetConstraintForPredicatContainerCommand();
			command.setPart((PredicateContainer)child.getModel());
			command.setLocation((Rectangle)constraint);
			return command;
		}
		return null;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getConstraintFor(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Object getConstraintFor(CreateRequest request) {
		IFigure figure = ((NavExpContainerEditPart)getHost()).getPrivateContentPane(request.getNewObject());

		Point where = request.getLocation().getCopy();
		Dimension size = request.getSize();

		figure.translateToRelative(where);
		figure.translateFromParent(where);
		where.translate(getLayoutOrigin(figure).getNegated());

		if(size == null || size.isEmpty())
			return getConstraintFor(where);
		else{		
			size = size.getCopy();
			figure.translateToRelative(size);
			figure.translateFromParent(size);	
			return getConstraintFor(new Rectangle(where, size));
		}
	}
	
	/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getLayoutOrigin()
		 */
	protected Point getLayoutOrigin(IFigure container) {
		XYLayout layout = (XYLayout)container.getLayoutManager();
		return layout.getOrigin(container);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getConstraintFor(org.eclipse.gef.requests.ChangeBoundsRequest, org.eclipse.gef.GraphicalEditPart)
	 */
	protected Object getConstraintFor(ChangeBoundsRequest request,GraphicalEditPart child) {
		IFigure figure = ((NavExpContainerEditPart)getHost()).getPrivateContentPane(child);
		
		Rectangle rect = new PrecisionRectangle(child.getFigure().getBounds());
		child.getFigure().translateToAbsolute(rect);
		Rectangle original = rect.getCopy();
		rect = request.getTransformedRectangle(rect);
		child.getFigure().translateToRelative(rect);
		rect.translate(getLayoutOrigin(figure).getNegated());

		if(request.getSizeDelta().width == 0 && request.getSizeDelta().height == 0){//move
			Rectangle cons = (Rectangle)getCurrentConstraintFor(child);
			rect.setSize(cons.width, cons.height);
		}else{//resize
			Dimension minSize = getMinimumSizeFor(child);
			if(rect.width < minSize.width){
				rect.width = minSize.width;
				if (rect.x > (original.right() - minSize.width))
					rect.x = original.right() - minSize.width;
			}
			if(rect.height < minSize.height){
				rect.height = minSize.height;
				if(rect.y > (original.bottom() - minSize.height))
					rect.y = original.bottom() - minSize.height;
			}
		}
		return getConstraintFor(rect);		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getAddCommand(org.eclipse.gef.Request)
	 */
	protected Command getAddCommand(Request generic) {
		if(generic instanceof ChangeBoundsRequest)return null;
		return super.getAddCommand(generic);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getMoveChildrenCommand(org.eclipse.gef.Request)
	 */
	protected Command getMoveChildrenCommand(Request request) {
		if(request instanceof ChangeBoundsRequest){
			ChangeBoundsRequest cbreq=(ChangeBoundsRequest) request;
			if(cbreq.getEditParts().get(0) instanceof ConditionEditPart)
				return null;
		}
		return super.getMoveChildrenCommand(request);
	}

}
