/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.editors;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.ui.actions.ActionFactory;

import vocleditor.action.PropertiesForSetClassifierRoleAction;
import vocleditor.action.PropertiesForClassifierRoleAction;
import vocleditor.action.PropertiesForVOCLContainerAction;
import vocleditor.action.VOCLExpNegateAction;

/**
 * This is the context menu provider for our editor.
 * 
 */
public class VOCLEditorMenuProvider extends ContextMenuProvider
{
    // the action registry
    private final ActionRegistry actionRegistry;

    /**
     * Creates a new NavExpEditorMenuProvider instance.
     * @param viewer - the viewer of the EditParts
     * @param actionRegistry - a class where all actions are registered
     */
    public VOCLEditorMenuProvider(EditPartViewer viewer, ActionRegistry actionRegistry) {
        super(viewer);
        this.actionRegistry = actionRegistry;
    }

    /**
     * Here, a contxt-menu is builded. This is done by adding
     * the actions to specific groups.
     */
    public void buildContextMenu(IMenuManager menuManager) {
    	
    	// Add the standard action groups
        GEFActionConstants.addStandardActionGroups(menuManager);

		// adding some of the actions (to different groups)
        appendActionToUndoGroup(menuManager, ActionFactory.UNDO.getId());
        appendActionToUndoGroup(menuManager, ActionFactory.REDO.getId());

        appendActionToEditGroup(menuManager, ActionFactory.COPY.getId());
        appendActionToEditGroup(menuManager, ActionFactory.PASTE.getId());
        appendActionToEditGroup(menuManager, ActionFactory.DELETE.getId());
        appendActionToEditGroup(menuManager,GEFActionConstants.DIRECT_EDIT);
        // Here, the AddAttributeAction is added (to Add-Group) 
        appendActionToAddGroup(
            menuManager,
            PropertiesForClassifierRoleAction.CLASSIFIER_PROPERTY);
            
        //AddCondition
		appendActionToAddGroup(
			 menuManager,
			PropertiesForVOCLContainerAction.CLASSIFIER_PROPERTY);    
			       
		// Here, the AddAttributeAction is added (to Add-Group) 
		appendActionToAddGroup(
			menuManager,
			PropertiesForSetClassifierRoleAction.CLASSIFIER_PROPERTY);
		
		//		VOCLExpNegateAction (to Add-Group) 
			 appendActionToAddGroup(
				 menuManager,
				 VOCLExpNegateAction.NAGATE_PROPERTY);
		
        appendAlignmentSubmenu(menuManager);

        appendActionToMenu(
            menuManager,
            ActionFactory.SAVE.getId(),
            GEFActionConstants.GROUP_SAVE);
    }

    /**
     * Appends the alignment submenu.
     * @param menuManager
     */
    private void appendAlignmentSubmenu(IMenuManager menuManager) {
        // Alignment Actions
        MenuManager submenu = new MenuManager("Align");

        IAction action =
            getActionRegistry().getAction(GEFActionConstants.ALIGN_LEFT);
        if (null != action && action.isEnabled())
            submenu.add(action);

        action = getActionRegistry().getAction(GEFActionConstants.ALIGN_CENTER);
        if (null != action && action.isEnabled())
            submenu.add(action);

        action = getActionRegistry().getAction(GEFActionConstants.ALIGN_RIGHT);
        if (null != action && action.isEnabled())
            submenu.add(action);

        action = getActionRegistry().getAction(GEFActionConstants.ALIGN_TOP);
        if (null != action && action.isEnabled())
            submenu.add(action);

        action = getActionRegistry().getAction(GEFActionConstants.ALIGN_MIDDLE);
        if (null != action && action.isEnabled())
            submenu.add(action);

        action = getActionRegistry().getAction(GEFActionConstants.ALIGN_BOTTOM);
        if (null != action && action.isEnabled())
            submenu.add(action);

        if (!submenu.isEmpty())
            menuManager.appendToGroup(GEFActionConstants.GROUP_REST, submenu);
    }

    /**
     * Returns the action registry.
     * @return the action registry
     */
    protected ActionRegistry getActionRegistry() {
        return actionRegistry;
    }

    /**
     * Appends the specified action to the specified menu group
     * @param actionId
     * @param menuGroup
     */
    private void appendActionToMenu(IMenuManager menu, String actionId, String menuGroup) {
        IAction action = getActionRegistry().getAction(actionId);
        if (null != action && action.isEnabled())
            menu.appendToGroup(menuGroup, action);
    }

    /**
     * Appends the specified action to the specified menu group
     * @param actionId
     * @param menuGroup
     */
    private void appendActionToUndoGroup(IMenuManager menu, String actionId)
    {
        IAction action = getActionRegistry().getAction(actionId);
        if (null != action && action.isEnabled())
            menu.appendToGroup(GEFActionConstants.GROUP_UNDO, action);
    }

    /**
     * Appends the specified action to the specified menu group
     * @param actionId
     * @param menuGroup
     */
    private void appendActionToEditGroup(IMenuManager menu, String actionId)
    {
        IAction action = getActionRegistry().getAction(actionId);
        if (null != action && action.isEnabled())
            menu.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
    }

    /**
     * Appends the specified action to the specified menu group
     * @param actionId
     * @param menuGroup
     */
    private void appendActionToAddGroup(IMenuManager menu, String actionId)
    {
        IAction action = getActionRegistry().getAction(actionId);
        if (null != action && action.isEnabled()) {
            menu.appendToGroup(GEFActionConstants.GROUP_ADD, action);
        }
    }

}
