/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.editors.tools;


import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.tools.CreationTool;
import org.eclipse.swt.widgets.Shell;

import vocleditor.commands.create.CreateBoolExpCommand;
import vocleditor.commands.create.CreateClassifierNodeCommand;
import vocleditor.commands.create.CreateConstraintCommand;
import vocleditor.commands.create.CreateLetExpCommand;
import vocleditor.commands.create.CreatePredicatContainerCommand;
import vocleditor.dialogs.CreateConstraintDialog;
import vocleditor.dialogs.CreateNewClassDialog;
import vocleditor.dialogs.CreatePredicatContainerDialog;
import vocleditor.dialogs.ExpressionDialog;


public class VOCLCreationTool extends CreationTool {

	/**
	 * Constructor
	 */
	public VOCLCreationTool() {super();}

	
	/**
	 * 
	 * @param aFactory
	 */
	public VOCLCreationTool(CreationFactory aFactory) {super(aFactory);}

	/**
	 * @see org.eclipse.gef.tools.AbstractTool#executeCurrentCommand()
	 */
	protected void executeCurrentCommand() {
		Command command=getCurrentCommand();
		Shell shell=this.getCurrentViewer().getControl().getShell();
		if((command instanceof CreateConstraintCommand) && command.canExecute()){
			if(!openDialogForConstraint((CreateConstraintCommand)command,shell))return;		
		}else if((command instanceof CreateClassifierNodeCommand) && command.canExecute()){
			if(!openDialogForClassifierNode((CreateClassifierNodeCommand)command,shell))return;
		}else if((command instanceof CreateBoolExpCommand) && command.canExecute()){
			if(!openDialogForBOOLorLETExpression((CreateBoolExpCommand)command,shell))return;
		}else if((command instanceof CreateLetExpCommand) && command.canExecute()){
			if(!openDialogForBOOLorLETExpression((CreateLetExpCommand)command,shell))return;
		}else if((command instanceof CreatePredicatContainerCommand) && command.canExecute()){
			if(!openDialogForSelectionContainer((CreatePredicatContainerCommand)command,shell))return;
		}
			
		super.executeCurrentCommand();
	}
	

	/**
	 * open the <code>CreateConstraintDialog</code> to create <code>VOCLConstraintEditPart</code>.
	 *
	 * @param command  active <code>CreateConstraintCommand</code>  
	 * @param shell	   the <code>Shell</code> 
	 * @return true if the Dialog was closed with OK Button,else false.
	 */
	private boolean openDialogForConstraint(CreateConstraintCommand command,Shell shell){
		CreateConstraintDialog d=new CreateConstraintDialog(shell);
		d.open();
		if(!d.isState())return false;
		//initilize command
		command.setContextKind(d.getKindC());
		command.setNameOfContext(d.getNameContext());
		command.setClassifier(d.isCl());
		command.setVariableName(d.getVariable());
		command.setParameter(d.getParameter());
		command.setTypeOfContext(d.getTypeC());
		command.setNameOfConstraint(d.getNameConstraint());
		
		return true;
	} 
	
	/**
	 * open the <code>CreateNewClassDialog</code> to create <code>SimpleClassifierRoleEditPart</code>.
	 *
	 * @param command  active <code>CreateSimpleClassifierRoleCommand</code>  
	 * @param shell	   the <code>Shell</code> 
	 * @return true if the Dialog was closed with OK Button,else false.
	 */
	private boolean openDialogForClassifierNode(CreateClassifierNodeCommand command,Shell shell){
		CreateNewClassDialog d=new CreateNewClassDialog(shell);
		d.open();
		if(!d.isStatus()) return false;
		
		//initilize command
		command.setNameOfClassifierNode(d.getVariableName());
		command.setTypeOfClassifierNode(d.getClassName());
		
		return true;
	} 
	
	/**
	 * open the <code>CreateNewClassDialog</code> to create <code>SimpleClassifierRoleEditPart</code>.
	 *
	 * @param command  active <code>CreateSimpleClassifierRoleCommand</code>  
	 * @param shell	   the <code>Shell</code> 
	 * @return true if the Dialog was closed with OK Button,else false.
	 */
	private boolean openDialogForBOOLorLETExpression(CreateBoolExpCommand command,Shell shell){
		ExpressionDialog d=new ExpressionDialog(shell,0);
		d.setName(command.getName());
		d.open();
		if(!d.isState()) return false;
		//initilize command
		command.setVertical(d.isVertical());
		command.setKind(d.getKind());

		return true;
	} 
	/**
	 * open the <code>CreateNewClassDialog</code> to create <code>SimpleClassifierRoleEditPart</code>.
	 *
	 * @param command  active <code>CreateLetExpCommand</code>  
	 * @param shell	   the <code>Shell</code> 
	 * @return true if the Dialog was closed with OK Button,else false.
	 */
	private boolean openDialogForBOOLorLETExpression(CreateLetExpCommand command,Shell shell){
		ExpressionDialog d=new ExpressionDialog(shell,0);
		d.setName(command.getName());
		d.open();
		if(!d.isState()) return false;
		//initilize command
		command.setVertical(d.isVertical());
		command.setKind(d.getKind());
		command.setVarname(d.getVarname());
		command.setVartype(d.getVartype());
		return true;
	} 

	private boolean openDialogForSelectionContainer(CreatePredicatContainerCommand command,Shell shell){
		CreatePredicatContainerDialog d=new CreatePredicatContainerDialog(shell,0);
		d.setValues(command.getValues());
		d.open();
		if(!d.isState()) return false;
		command.setValues(d.getValues());
		return true;
	} 


}
