/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.figure;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

/**
 * @author vilapower
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 * 
 *AbstractNodeFigure provides the AnchorPoints for the ClassifierRoles the classColor and the classFont
 * 
 *  (c) Copyright group Vilapower
 * 
 * @author vilapower
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger
 */

public abstract class AbstractNodeFigure extends Figure{
	
  	protected AbstractNodeFigure figure;
  	protected int width;
  	protected int height;
  	
	
	// the anchors for in- and outgoing Connections (Associations)
	protected List incomingConnectionAnchors;
	protected List outgoingConnectionAnchors;
	protected Point mousePoint = new Point(0,0);


	/**
	 * Constructor of the ClassifierRoleFigure:
	 * Creates a Figure for a class with a class-name,
	 * a variable-name (optional - may be empty, but not null),
	 * and attributes and place them inside a UML-like
	 * box for classes.
	 * @param shell     - the shell where the figure is drawn on
	 * @param varname   - the variable-name for this class-instance
	 * @param classname - the class name
	 */
	public AbstractNodeFigure(String varname,String classname){			
		// place everything inside a XYLayout
		// and a LineBorder
		XYLayout layout = new XYLayout();
		setLayoutManager(layout);
		setOpaque(true);		

		// add connection anchors
		incomingConnectionAnchors = new ArrayList();
		outgoingConnectionAnchors = new ArrayList();
				
	}
	
	/**
	 * getDim detects the extensions of the ClassifierRole
	 * @return the Dimension of the ClassifierRole
	 */
	private Dimension getDim(){
	
	// detect the dimension of ClassifierRole
	Dimension dF;
	if(this instanceof SetClassifierRoleFigure) {
		SetClassifierRoleFigure f = (SetClassifierRoleFigure)this;
		dF = f.getPreferredSize();
	}
	else {
		ClassifierRoleFigure f = (ClassifierRoleFigure)this;
		dF = f.figure.getPreferredSize();
	}
	return dF;	
	}

	/**
	 * add the Anchors to the SimpleClassifierRoleFigure
	 *
	 */	
	protected void createVoclEditorAnchors(){
		
		Dimension dF = getDim();
		// Hoehe und Breite
		width = dF.width;
		height = dF.height;
					
		VoclAnchor tempA; // = new VoclAnchor(this);

		// mitte links					(0)
		tempA = new VoclAnchor(this);
		tempA.setOffset(0, height/2);
		tempA.setId(0);
		incomingConnectionAnchors.add(tempA);
		outgoingConnectionAnchors.add(tempA);

		// unten links					(1)
		tempA = new VoclAnchor(this);
		tempA.setOffset(0, height - 1);
		tempA.setId(1);
		incomingConnectionAnchors.add(tempA);
		outgoingConnectionAnchors.add(tempA);

		// unten mitte 1				(2)
		tempA = new VoclAnchor(this);
		tempA.setOffset(width/3, height -1);
		tempA.setId(2);
		incomingConnectionAnchors.add(tempA);
		outgoingConnectionAnchors.add(tempA);

		// unten mitte 2				(3)
		tempA = new VoclAnchor(this);
		tempA.setOffset(width*2/3, height -1);
		tempA.setId(3);
		incomingConnectionAnchors.add(tempA);
		outgoingConnectionAnchors.add(tempA);
		
		//		mitte 1 oben			(4)
		tempA = new VoclAnchor(this);
		tempA.setOffset(width/3, 0);
		tempA.setId(4);
		incomingConnectionAnchors.add(tempA);
		outgoingConnectionAnchors.add(tempA);	
	
		// mitte 2 oben				(5)
		tempA = new VoclAnchor(this);
		tempA.setOffset(width*2/3, 0);
		tempA.setId(5);
		incomingConnectionAnchors.add(tempA);
		outgoingConnectionAnchors.add(tempA);
					
		// produce extra Anchors for SimpleClassifierRoleFigures
		if (this instanceof ClassifierRoleFigure)			
			createSimpleClassifierAnchors();				
			
	}

	/**
	 * creates more Anchors especially for SimpleClassifierFigures
	 */
	private void createSimpleClassifierAnchors() {
			
			VoclAnchor tempA; // = new VoclAnchor(this);
					
			//		links oben			(6)
			tempA = new VoclAnchor(this);
			tempA.setOffset(0, 0); 
			tempA.setId(6);
			incomingConnectionAnchors.add(tempA);
			outgoingConnectionAnchors.add(tempA);
	 	
	
			 // rechts oben 			(7)
			 tempA = new VoclAnchor(this);
			 tempA.setOffset(width - 1, 0);
			 tempA.setId(7);
			 incomingConnectionAnchors.add(tempA);
			 outgoingConnectionAnchors.add(tempA);					
	
			// mitte rechts 				(8)
			tempA = new VoclAnchor(this);
			tempA.setOffset(width -1, height/2);
			tempA.setId(8);
			incomingConnectionAnchors.add(tempA);
			outgoingConnectionAnchors.add(tempA);
			
			// unten rechts				(9)
			tempA = new VoclAnchor(this);
			tempA.setOffset(width -1, height -1);
			tempA.setId(9);
			incomingConnectionAnchors.add(tempA);
			outgoingConnectionAnchors.add(tempA);
					
				
					
	}

	/**
	 * Retrieves the SourceConnectionAnchor of this 
	 * figure.
	 * @return - the source-anchor
	 */
	public List getSourceConnectionAnchors(){
		return outgoingConnectionAnchors;
		}
	
	/**
	 * Retrieves the TargetConnectionAnchor of this 
	 * figure.
	 * @return - the target-anchor
	 */
	public List getTargetConnectionAnchors(){
		return incomingConnectionAnchors;
		}
	
	/**
	 * Detection of the closest source connection anchor around the mouse pointer
	 * @return the anchor which is closest to the actual mouse position
	 */
	public VoclAnchor getSourceConnectionAnchorAt() {		
		return getClosestConnectionAnchorAt(mousePoint, getSourceConnectionAnchors());
	}
	
	/**
	 * Detection of the closest target connection anchor around the mouse pointer
	 * @return the anchor which is closest to the actual mouse position
	 */
	public VoclAnchor getTargetConnectionAnchorAt() {		
		return getClosestConnectionAnchorAt(mousePoint, getTargetConnectionAnchors());
	}
	
	
	/**
	 * Calculating the minimal distant anchor regarding to the mouse pointer
	 * @param p
	 * @param connectionAnchors
	 * @return the closest anchor around the point p
	 */
	static public VoclAnchor getClosestConnectionAnchorAt(Point p, List connectionAnchors) {
		
		// initializing variables
		VoclAnchor closest = null;
		long min = Long.MAX_VALUE;
		int anz_Anchors = connectionAnchors.size();
		Point p2 = new Point(0,0);
		
		// checking each anchor point and taking the best one
		for (int i = 0; i < anz_Anchors; i++) {
			VoclAnchor a = (VoclAnchor) connectionAnchors.get(i);
			
			// absolut coordinates of the anchor 
			p2 = a.getLocation(null);
			
			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = a;
			}
		}
		return closest;
	}

	/**
	 * This method refreshes the Position of the AnchorPoints at the Classifier Role as soon as there has been occured a changing of the dimension
	 */
	protected void refreshAnchorPoints() {
		
		Dimension dF = getDim();
		// Hoehe und Breite
		width = dF.width;
		height = dF.height;
		
		// down contains the offset: the arrow start to the classifier rectangle top left
		int down = 0;
		if(this instanceof ClassifierRoleFigure) {
			ClassifierRoleFigure f = (ClassifierRoleFigure) this;
			if (f.isMethod()) down = ClassifierRoleFigure.ARROW_LENGTH; 
		}
		
		int nr = 0;
	
				// mitte links					(0)
				// unten links					(1)
				// unten mitte 1				(2)
				// unten mitte 2				(3)
				// oben mitte 1				(4)
				// oben mitte 2				(5)					

		//		mitte links
		((VoclAnchor) incomingConnectionAnchors.get(0)).setOffset(0, down + height/2);
		((VoclAnchor) outgoingConnectionAnchors.get(0)).setOffset(0, down + height/2);	
		
		//		unten links
		((VoclAnchor) incomingConnectionAnchors.get(1)).setOffset(0, down + height - 1);
		((VoclAnchor) outgoingConnectionAnchors.get(1)).setOffset(0, down + height - 1);

		//		unten mitte 1
		((VoclAnchor) incomingConnectionAnchors.get(2)).setOffset(width/3, down + height -1);
		((VoclAnchor) outgoingConnectionAnchors.get(2)).setOffset(width/3, down + height -1);
		
		//		unten mitte 2
		((VoclAnchor) incomingConnectionAnchors.get(3)).setOffset(width*2/3, down + height -1);
		((VoclAnchor) outgoingConnectionAnchors.get(3)).setOffset(width*2/3, down + height -1);
		
		//		mitte 1 oben
		((VoclAnchor) incomingConnectionAnchors.get(4)).setOffset(width/3,down + 0);
		((VoclAnchor) outgoingConnectionAnchors.get(4)).setOffset(width/3,down + 0);

		// mitte 2 oben
		((VoclAnchor) incomingConnectionAnchors.get(5)).setOffset(width*2/3,down + 0);
		((VoclAnchor) outgoingConnectionAnchors.get(5)).setOffset(width*2/3,down + 0);
		
		//		refresh extra Anchors for SimpleClassifierRoleFigures
			 if (this instanceof ClassifierRoleFigure)			
				 {refreshExtraSimpleClassifierAnchors(down);} 
				 
	}

	/**
	 * Refreshing the extra points
	 */
	private void refreshExtraSimpleClassifierAnchors(int down) {
		 //	links oben				(6)
		 // rechts oben 			(7)
		 // mitte rechts 				(8)
		 // unten rechts				(9)
		
		//		links oben
		((VoclAnchor) incomingConnectionAnchors.get(6)).setOffset(0,down + 0);
		((VoclAnchor) outgoingConnectionAnchors.get(6)).setOffset(0,down + 0);
		
		//		rechts oben 
		((VoclAnchor) incomingConnectionAnchors.get(7)).setOffset(width - 1,down + 0);
		((VoclAnchor) outgoingConnectionAnchors.get(7)).setOffset(width - 1,down + 0);

		//		mitte rechts 
		((VoclAnchor) incomingConnectionAnchors.get(8)).setOffset(width -1,down + height/2);
		((VoclAnchor) outgoingConnectionAnchors.get(8)).setOffset(width -1,down + height/2);

		//		unten rechts
		((VoclAnchor) incomingConnectionAnchors.get(9)).setOffset(width -1,down + height -1);
		((VoclAnchor) outgoingConnectionAnchors.get(9)).setOffset(width -1,down + height -1);

	}

	/**
	 * @return the mouse point
	 */
	public Point getMousePoint() {return mousePoint;}

	/**
	 * @param point
	 */
	public void setMousePoint(Point point) {mousePoint = point;}

	/**
	 * refreshing the mouse point according to the request
	* @param point
	*/
	public void setMousePoint(Request req) {
		Point p =new Point(0,0);
		if  (req instanceof CreateConnectionRequest) {	
			CreateConnectionRequest r = (CreateConnectionRequest) req;			
			p = r.getLocation();
		}else if (req instanceof ReconnectRequest) {
			ReconnectRequest r2 = (ReconnectRequest) req;
			p = r2.getLocation();
		}
		setMousePoint(p);
	}

	/** 
	 * refreshing the anchor points every time when a figure has changed its extensions 
	 */
	public void validate() {
		refreshAnchorPoints();
		super.validate();
		
	}
	
	/**
	 * Overwritten method, indicates that a local
	 * coordinate system is used.
	 */
	protected boolean useLocalCoordinates() {return true;}


}
