/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.figure;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.TitleBarBorder;
import org.eclipse.swt.graphics.Color;

import vocleditor.editors.VOCLEditorConstants;



public class BoolExpFigure extends Figure{
	
	protected double proportion=0.5;
	protected boolean vertical=true;
	private static Color titleColor = new Color(null,190,190,225);	
	
	protected Panel firstPanel;
	protected Panel secondPanel;
	protected VOCLPanel panel;
	protected String type;
	
	protected boolean negated =false;
	Panel notPanel;
	Panel notBorderPanel;
	TitleBarBorder border;
	LineBorder notBorder = new LineBorder(ColorConstants.white,0);
	
	
	public BoolExpFigure(boolean v,String type) {
		super();
		this.type = type;
		setVertical(v);
		setLayoutManager(new BorderLayout());
		setBorder(new LineBorder(ColorConstants.black,2));
		
		panel=new VOCLPanel();
		panel.setLayoutManager(new VOCLBorderLayout());
		panel.setBorder(new LineBorder(VOCLEditorConstants.titleColor,3));
		
		
		
		Panel mainpanel=new Panel();
		mainpanel.setLayoutManager(new BorderLayout());
		init(mainpanel);
		
		if(type.trim().length()!=0){		
			border=new TitleBarBorder(type);
			border.setBackgroundColor(VOCLEditorConstants.titleColor);
			border.setTextAlignment(PositionConstants.CENTER);
			border.setFont(VOCLEditorConstants.font);
			mainpanel.setBorder(border);
		}		
			
		add(mainpanel, BorderLayout.CENTER);
	
				
	}

	public void init(Panel mainpanel){
		firstPanel=new Panel();
		firstPanel.setLayoutManager(new BorderLayout());
		secondPanel=new Panel();
		secondPanel.setLayoutManager(new BorderLayout());
		if(isVertical()){
			panel.add(firstPanel,BorderLayout.LEFT);
			panel.add(secondPanel,BorderLayout.RIGHT);	
		}else{
			panel.add(firstPanel,BorderLayout.TOP);
			panel.add(secondPanel,BorderLayout.BOTTOM);				
		}
		mainpanel.add(panel,BorderLayout.CENTER);		
	}

	/**
	 * @return
	 */
	public double getProportion() {
		return panel.getProportion();
	}

	/**
	 * @param d
	 */
	public void setProportion(double d) {
		panel.setProportion(d);	
		firstPanel.validate();
		secondPanel.validate();

	}

	/**
	 * @return
	 */
	public boolean isVertical() {
		return vertical;
	}

	/**
	 * @param b
	 */
	public void setVertical(boolean b) {
		vertical = b;
	}

	/**
	 * @return
	 */
	public VOCLPanel getPanel() {
		return panel;
	}

	/**
	 * @param panel
	 */
	public void setPanel(VOCLPanel panel) {
		this.panel = panel;
	}

	/**
	 * @return
	 */
	public Panel getFirstPanel() {
		return firstPanel;
	}

	/**
	 * @return
	 */
	public Panel getSecondPanel() {
		return secondPanel;
	}

	/**
	 * @param panel
	 */
	public void setFirstPanel(Panel panel) {
		firstPanel = panel;
	}

	/**
	 * @param panel
	 */
	public void setSecondPanel(Panel panel) {
		secondPanel = panel;
	}

	
	/**
	 * @return
	 */
	public boolean isNegated() {
		return negated;
	}

	/**
	 * @param b
	 */
	public void setNegated(boolean b) {
		negated = b;
		PanelLayout();
	}

	public void PanelLayout() {
	
			if (negated) {	
				border.setLabel("NOT[ " + type + "]");
				border.setBackgroundColor(VOCLEditorConstants.negateColor);
				((LineBorder)panel.getBorder()).setColor(VOCLEditorConstants.negateColor);		
			}
			else {
				border.setLabel(type);
				border.setBackgroundColor(VOCLEditorConstants.titleColor);
				((LineBorder)panel.getBorder()).setColor(VOCLEditorConstants.titleColor);		
			}
	repaint();
	}




}
