/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.figure;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Color;


public class ShadowClassFigure extends Figure {

	// the label for the class and variable
	// private Label  classLabel;
	private ClassifierRoleLabel classFigure;
	private Figure rightFigure;
	private Figure topFigure;
	private Figure lowerFigure;
	private String className;
	private String varName;
	
	
	public ShadowClassFigure(String varName, String className) {		
		this.varName = varName;
		this.className = className;
		
 		classFigure = new ClassifierRoleLabel(varName, className);
 
		// and a LineBorder
		ToolbarLayout layout = new ToolbarLayout();
		setLayoutManager(layout);
		setOpaque(true);
				
		
		// our Dimension
		Dimension dim = classFigure.getPreferredSize(0,0);
		topFigure = this.getTopFigure(dim);
		this.add(topFigure);
		
		lowerFigure = new Figure();
		lowerFigure.setLayoutManager(new ToolbarLayout(true));
		lowerFigure.add(classFigure);
		rightFigure = this.getRightFigure(dim);
		lowerFigure.add(this.getRightFigure(dim));		
		this.add(lowerFigure);
	}
	
	/**
	 * Retrieving the wanted Dimension for this figure.
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		Dimension dim1 = classFigure.getPreferredSize();
		return new Dimension(dim1.width+6, dim1.height+6);
	}
	
	/**
	 * Retrieves a Figure for the upper part of the
	 * ShadowClassFigure
	 * @param dim - the dimension of the classLabel
	 * @return - the figure (white,black rects)
	 */
	public Figure getTopFigure(Dimension dim) {
		//	creating the topFigure
		 Figure topFigure = new Figure();		
		 topFigure.setLayoutManager(new ToolbarLayout(true));
		
		 // the upper left white rectangle
		 Figure whiteRect = new Figure();
		 whiteRect.setBackgroundColor(new Color(null, 255,255,255));
		 whiteRect.setPreferredSize(new Dimension(6,6));
		 topFigure.add(whiteRect);

		 Figure greyRect = new RectangleFigure();
		 greyRect.setBackgroundColor(new Color(null, 0,0,0));
		 greyRect.setForegroundColor(new Color(null, 0,0,0));
		 greyRect.setPreferredSize(new Dimension(dim.width,6));
		 topFigure.add(greyRect);
		 return topFigure;
	}
	
	/**
	 * Retrieves a Figure for the right side of
	 * the ShadowClassFigure (black,white rects)
	 * @param dim - the Dimension of the classlabel
	 * @return - the Figure with a Shadow
	 */
	public Figure getRightFigure(Dimension dim) {		
		 // the right left white rectangle
		 Figure rightFigure = new Figure();
		 rightFigure.setLayoutManager(new ToolbarLayout());
		
		 Figure greyRect2 = new RectangleFigure();
		 greyRect2.setBackgroundColor(new Color(null, 0,0,0));
		 greyRect2.setForegroundColor(new Color(null, 0,0,0));
		 greyRect2.setPreferredSize(new Dimension(6,dim.height-6));
		 rightFigure.add(greyRect2);

		 Figure whiteRect2 = new Figure();
		 whiteRect2.setBackgroundColor(new Color(null, 255,255,255));
		 whiteRect2.setForegroundColor(new Color(null, 255,255,255));
		 whiteRect2.setPreferredSize(new Dimension(6,6));
		 rightFigure.add(whiteRect2);
		 return rightFigure;
	}
	
	/**
	 * Setting the Label for this ClassifierFigure
	 * @param varName - the variables name
	 * @param className - the classifiers name
	 */
	public void setClassifierText(String varName, String className) {
		this.varName = varName;
		this.className = className;
		classFigure.setClassVarName(varName);
		this.updateFigure();
	}
	
	/**
	 * Only setting the variables name
	 * @param name - the variables name
	 */
	public void setVarName(String name) {
		setClassifierText(name, this.className);
		this.updateFigure();
	}
	
	/**
	 * This method is called to calculate the 
	 * new sizes for the upper and right figures
	 * elements.
	 */
	public void updateFigure() {
		this.removeAll();
		Dimension dim = classFigure.getPreferredSize();
		topFigure = this.getTopFigure(dim);
		this.add(topFigure);
		
		lowerFigure = new Figure();
		lowerFigure.setLayoutManager(new ToolbarLayout(true));
		lowerFigure.add(classFigure);
		rightFigure = this.getRightFigure(dim);
		lowerFigure.add(rightFigure);		
		this.add(lowerFigure);
	}
	
	/**
	 * Delegates the addAttribute Command to the
	 * classFigure (which is a SimpleClassifierRoleFigure)
	 * @param name - an attributes name
	 * @param type - an attributes type
	 * @param value - an attributes value
	 */
	public void addAttribute(String name, String type, String value, boolean isValue) {
		classFigure.addAttribute(name, type, value, isValue);
		// update the figures
		updateFigure();		
	}
	
	public void removeAttribute(int i) {
		classFigure.removeAttribute(i);
		updateFigure();
	}
	
	public void removeAllAttributes() {
		classFigure.removeAllAttributes();
	}
}
