/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.model;

import java.util.Map;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

import VOCLModel.*;




/**
 * This class implements the CreationFactory used by the CreationTool. It in turn
 * uses the EMF-generated factories to create the model instances
 *
 */
public class ModelCreationFactory  implements CreationFactory {

	private java.lang.Class targetClass;
	
	public ModelCreationFactory( java.lang.Class targetClass ) {
		this.targetClass = targetClass;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
	 */
	public Object getNewObject() {
		Map registry = EPackage.Registry.INSTANCE;
		String navURI = VOCLModel4Package.eNS_URI;
		VOCLModel4Package navExpPackage =
		(VOCLModel4Package) registry.get(navURI);
		VOCLModel4Factory factory = navExpPackage.getVOCLModel4Factory();
			
		Object		result = null;	
		if( targetClass.equals( VOCLDiagramm.class ) ) {
			result = factory.createVOCLDiagramm();
		}else if( targetClass.equals( Constraint.class ) ) {
			result = factory.createConstraint();
		}else if( targetClass.equals( NavExpContainer.class ) ) {
			result = factory.createNavExpContainer();
		}else if( targetClass.equals( BoolExp.class ) ) {
			result = factory.createBoolExp();
		}else if( targetClass.equals( LetExp.class ) ) {
			result = factory.createLetExp();
		}else if( targetClass.equals( IfExp.class ) ) {
			result = factory.createIfExp();
		}else if( targetClass.equals( AssociationEndRoleSource.class ) ) {
			result = factory.createAssociationEndRoleSource();
		}else if (targetClass.equals(AssociationEndRoleTarget.class)){
			result = factory.createAssociationEndRoleTarget();
		}else if (targetClass.equals(AssociationEndRoleSource.class)){
			result = factory.createAssociationEndRoleSource();
		}else if (targetClass.equals(AssociationEndRoleTarget.class)){
			result = factory.createAssociationEndRoleTarget();
		}else if (targetClass.equals(AssociationNav.class)){
			result = factory.createAssociationNav();
		}else if (targetClass.equals(AssociationIsIn.class)){
			result = factory.createAssociationIsIn();
		}else if (targetClass.equals(Attribute.class)){
			result = factory.createAttribute();
		}else if (targetClass.equals(ClassifierRole.class)){
			result = factory.createClassifierRole();
		}else if (targetClass.equals(SetClassifierRole.class)){
				result = factory.createSetClassifierRole();
		}else if (targetClass.equals(VariableDeclaration.class)){
			result = factory.createVariableDeclaration();
		}else if (targetClass.equals(PredicateContainer.class)){
			result = factory.createPredicateContainer();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
	 */
	public Object getObjectType() {
		return targetClass;
	}



}
